-- Useful Number-theoretical function collection
-- (C) Konstantin Tretyakov.

module Numtheory where

-- Erathosphene's sieve
sieve (x:xs) = [t | t<-xs, t`mod`x /= 0]

-- Primes obtained with sieve
primes = map head (iterate sieve [2..])


factor x = factorlist x primes 
	where factorlist 1 _ = []
	      factorlist 0 _ = []
	      factorlist x (p:ps)
		| x < 0		= factorlist (-x) (p:ps)
		| x`mod`p == 0	= (p,u):(factorlist nx ps)
		| otherwise	= factorlist x ps
		where	u = findpdegree x p
			nx = removep x
			removep x = if (x`mod`p == 0) then
					removep (x`div`p) else x

findpdegree x p
	| x`mod`p == 0	= 1+findpdegree (x`div`p) p
	| otherwise	= 0

factor2 x = factorlist x primes
	where factorlist 1 _ = []
	      factorlist 0 _ = []
	      factorlist x (p:ps)
		| x < 0		= factorlist (-x) (p:ps)
		| x`mod`p == 0	= p:(factorlist (x`div`p) (p:ps))
		| otherwise 	= factorlist x ps

defactor list = product (map (uncurry power) list)

power a b = if (b==0) then 1 else a*power a (b-1)

phi x = product (map (\(a,b)->(a-1)*(power a (b-1))) (factor x))

moebius x = product (map (\(a,b)->if (b>1) then 0 else (-1)) (factor x))

divisors n = [ t | t<-[1..n], n`mod`t==0 ] 

ristsum x = sum (map (\c -> (ord c - ord '0')) (show x))

-- Theorem 1: divphisum n == n
divphisum n = sum (map phi (divisors n))

-- Theorem 2: divmobsum n = 0
divmobsum n = sum (map moebius (divisors n))


------ Given relatively prime a and b returns (u,v) such
------ that au + bv = 1
euclid :: (Integral a) => a -> a -> (a,a)
euclid a b
   | gcd a b /= 1   = error "euclid - Parameters must be relatively prime!"
   | a < 1 || b < 1 = error "euclid - Parameters must be positive!"
   | a < b          = swap (euclid b a)
   | a `mod` b == 1 = (1, - (a `div` b))
   | otherwise      = (v, (u - (v*k)))
   where swap (a,b) = (b,a)
         (u, v)     = euclid b c
         k          = a `div` b
         c          = a `mod` b


-- Gets n and a returns inverse(a) modulo n
modInverse n a = fst (euclid a n)

-- Returns a^x mod n. Works for negative x if a is invertible
modPow n a 0 = 1
modPow n a x
    | x < 0           = modPow n (modInverse n a) (-x)
    | x `mod` 2 == 0  = t * t `mod` n
    | otherwise       = t * t * a `mod` n
    where t = modPow n a (x `div` 2)

-- Accepts a list [(a1, n1),(a2, n2), ...] 
-- and returns the solution found by the china remainder theorem
china list = sum [ (a i)*(k i)*(m i) | i <- [0..length(list)-1] ] `mod` nn
    where 
          n i = snd (list!!i)
          a i = fst (list!!i)
          nn = product (map snd list)
          m i = nn `div` (n i)
          k i = modInverse (n i) (m i)


--- Evaluates the polynomial given as a list of coefficients at a given point
--- Coefficients are given in a list as [a_0, a_1, ...] i.e. least first.
polynomial [] x = 0
polynomial (a:as) x = x * (polynomial as x) + a

